/******************************************************
*
*    COPYRIGHT (C) M.BOS 1999 , for details see copying
*
*
*    gen_spec.c
*
*    generates X-ray tube spectrum based
*    on Pella's algorithms
*    P.A.Pella, L. Feng, J.A.Small
*    X-ray spectrom. 14 (1985) 125
*
*    coded by
*    M.Bos
*    oct 1997
*
*    adjusted characteristic line treatment
*    Oct 1998
*
*    attempt to adjust influence charac. lines from tube
*    spectrum to correct lambda-range, i.e. not extending
*    to longer wavelengths.
*    Feb 1999
*
*    charac.line intensity spread over 2 spectrumpoints
*    jan 25th 2003
*
*
*    corrected tube filter code - March 12th  2004
*
*    new version that reads spectrum from file Dec 16th 2008
*    first entry = nr of points
*    table : per line  keV  Intensity pair
********************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "xrfluor.h"

extern elem_list lijst_els;
extern emudata abstabel ;
extern double pellas_consts[4][3];
struct spectrum_pair  *gen_spec( void)
{
   int i;
   FILE *fpspectrum;
   struct spectrum_pair *spectrum ;
   int nr_points;
   int retvalscan ;
   double labda;
   

  if ((fpspectrum=fopen("spectrum.dat","r"))==NULL)
   {
      printf("Cannot open file spectrum.dat \n");
      exit(1);
   }  /* */

   fscanf(fpspectrum,"%d", &nr_points);   /* read number of spectrum points */
   PNTS_SPECTRUM=nr_points;
   spectrum= malloc(nr_points*sizeof(struct spectrum_pair));
   for (i=0; i < nr_points; i++)
   {
      retvalscan=fscanf(fpspectrum,"%lg", &labda);
      if ((retvalscan == EOF) || (retvalscan !=1))
      {
        printf("not enough data in spectrum.dat while reading lambda[%d]\n", i);
        exit(1);
      }
      (spectrum[i]).lambda=12.396/labda ;
      retvalscan= fscanf(fpspectrum,"%le", &((spectrum[i]).intens)); 
      if ((retvalscan == EOF) || (retvalscan !=1))
      {
        printf("not enough data in spectrum.dat while reading intensity[%d]\n", i);
        exit(1);
      }
   }
  fclose(fpspectrum); 
 /*  printf("net voor return in gen-spec \n"); /* */
   return(spectrum);
}

